==> csdk-runner-1.0.3-sources.jar: csdk/v1_0/runner/filesystem/CSDKLocalFileLock.java <==
package csdk.v1_0.runner.filesystem;

import java.awt.Window;
import java.io.File;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.nio.channels.FileChannel;
import java.nio.channels.FileLock;

import tecgraf.javautils.LNG;
import tecgraf.javautils.core.io.FileUtils;
import tecgraf.javautils.gui.Task;
import csdk.v1_0.api.filesystem.IFile;
import csdk.v1_0.api.filesystem.IFileLock;

/**
 * Representa um lock de um {@link IFile arquivo}.
 */
public class CSDKLocalFileLock implements IFileLock {

  /**
   * Lock real.
   */
  private final FileLock lock;

  /**
   * Canal para manipulao do arquivo.
   */
  private FileChannel channel;

  /**
   * Estado do lock.
   */
  private LockStatus status;

  /**
   * Construtor.
   * 
   * @param file o arquivo a ser bloqueado.
   * @param shared determina se o lock deve ser do tipo compartilhado.
   * @param window janela me da ao de lock.
   * @throws Exception em caso de erro ao criar o lock.
   */
  public CSDKLocalFileLock(final File file, final boolean shared,
    final Window window) throws Exception {
    if (file.isDirectory()) {
      throw new Exception("A directory cannot be locked ("
        + file.getAbsolutePath() + ")");
    }
    Task<FileLock> task = new Task<FileLock>() {
      @Override
      protected void performTask() throws Exception {
        FileLock fileLock = _lock(shared, window, file);
        setResult(fileLock);
      }
    };
    String prefix = CSDKLocalFileLock.class.getSimpleName();
    String key;
    if (shared) {
      key = ".shared.lock.message";
    }
    else {
      key = ".exclusive.lock.message";
    }
    String message = LNG.get(prefix + key);
    task.execute(window, null, message);
    this.lock = task.getResult();
    setLockStatus();
    if (task.wasCancelled()) {
      releaseLock(window);
    }
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public LockStatus getLockStatus() {
    return status;

  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void releaseLock(final Window window) {
    Task<Void> task = new Task<Void>() {
      @Override
      protected void performTask() throws Exception {
        try {
          _releaseLock(lock);
        }
        finally {
          FileUtils.close(channel);
        }
      }
    };
    String prefix = CSDKLocalFileLock.class.getSimpleName();
    String message = LNG.get(prefix + ".release.lock.message");
    if (task.execute(window, null, message)) {
      this.status = LockStatus.LOCK_RELEASED;
    }
  }

  /**
   * Obtm o lock do arquivo.
   * 
   * @param shared determina se o lock deve ser do tipo compartilhado.
   * @param window janela me da ao de lock.
   * @param file arquivo a ser bloqueado.
   * @return o lock do arquivo.
   * @throws IOException em caso de erro ao bloquear o arquivo.
   */
  @SuppressWarnings("resource")
  private FileLock _lock(final boolean shared, final Window window,
    final File file) throws IOException {
    RandomAccessFile randomAccessFile = null;
    try {
      final String mode = shared ? "r" : "rws";
      randomAccessFile = new RandomAccessFile(file, mode);
      this.channel = randomAccessFile.getChannel();
      return this.channel.tryLock(0, Long.MAX_VALUE, shared);
    }
    catch (Exception e) {
      FileUtils.close(randomAccessFile);
      return null;
    }
  }

  /**
   * Libera o lock.
   * 
   * @param fileLock o lock.
   * @throws IOException em caso de erro ao liberar o lock;
   */
  private void _releaseLock(final FileLock fileLock) throws IOException {
    if (fileLock != null && fileLock.isValid()) {
      fileLock.release();
    }
  }

  /**
   * Determina o estado do lock.
   */
  private void setLockStatus() {
    if (lock != null) {
      if (this.lock.isShared()) {
        this.status = LockStatus.LOCK_SHARED;
      }
      else {
        this.status = LockStatus.LOCK_EXCLUSIVE;
      }
    }
    else {
      this.status = LockStatus.LOCK_DENIED;
    }
  }
}

==> csdk-runner-1.0.4-sources.jar: csdk/v1_0/runner/filesystem/CSDKLocalFileLock.java <==
package csdk.v1_0.runner.filesystem;

import java.awt.Window;
import java.io.File;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.nio.channels.FileChannel;
import java.nio.channels.FileLock;
import java.nio.channels.OverlappingFileLockException;

import tecgraf.javautils.LNG;
import tecgraf.javautils.core.io.FileUtils;
import tecgraf.javautils.gui.Task;
import csdk.v1_0.api.filesystem.IFile;
import csdk.v1_0.api.filesystem.IFileLock;
import csdk.v1_0.runner.CSDKLogger;

/**
 * Representa um lock de um {@link IFile arquivo}.
 */
public class CSDKLocalFileLock implements IFileLock {

  /**
   * Lock real.
   */
  private final FileLock lock;

  /**
   * Canal para manipulao do arquivo.
   */
  private FileChannel channel;

  /**
   * Construtor.
   * 
   * @param file o arquivo a ser bloqueado.
   * @param shared determina se o lock deve ser do tipo compartilhado.
   * @param window janela me da ao de lock.
   * @throws Exception em caso de erro ao criar o lock.
   */
  public CSDKLocalFileLock(final File file, final boolean shared,
    final Window window) throws Exception {
    if (file.isDirectory()) {
      throw new Exception("A directory cannot be locked ("
        + file.getAbsolutePath() + ")");
    }
    Task<FileLock> task = new Task<FileLock>() {
      @Override
      protected void performTask() {
        FileLock fileLock = _lock(shared, window, file);
        setResult(fileLock);
      }
    };
    String prefix = CSDKLocalFileLock.class.getSimpleName();
    String key;
    if (shared) {
      key = ".shared.lock.message";
    }
    else {
      key = ".exclusive.lock.message";
    }
    String message = LNG.get(prefix + key);
    task.execute(window, null, message);
    this.lock = task.getResult();
    if (task.wasCancelled()) {
      releaseLock(window);
    }
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public LockStatus getLockStatus() {
    if (lock != null) {
      if (this.lock.isValid()) {
        if (this.lock.isShared()) {
          return LockStatus.LOCK_SHARED;
        }
        else {
          return LockStatus.LOCK_EXCLUSIVE;
        }
      }
      else {
        return LockStatus.LOCK_RELEASED;
      }
    }
    else {
      return LockStatus.LOCK_DENIED;
    }
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void releaseLock(final Window window) {
    Task<Void> task = new Task<Void>() {
      @Override
      protected void performTask() throws Exception {
        try {
          _releaseLock(lock);
        }
        finally {
          FileUtils.close(channel);
        }
      }
    };
    String prefix = CSDKLocalFileLock.class.getSimpleName();
    String message = LNG.get(prefix + ".release.lock.message");
    task.execute(window, null, message);
  }

  /**
   * Obtm o lock do arquivo.
   * 
   * @param shared determina se o lock deve ser do tipo compartilhado.
   * @param window janela me da ao de lock.
   * @param file arquivo a ser bloqueado.
   * @return o lock do arquivo.
   */
  @SuppressWarnings("resource")
  private FileLock _lock(final boolean shared, final Window window,
    final File file) {
    RandomAccessFile randomAccessFile = null;
    try {
      final String mode = shared ? "r" : "rws";
      randomAccessFile = new RandomAccessFile(file, mode);
      this.channel = randomAccessFile.getChannel();
      return this.channel.tryLock(0, Long.MAX_VALUE, shared);
    }
    catch (OverlappingFileLockException e) {
      logSevere("Lock for file " + file.getPath()
        + " already exists in current VM");
      FileUtils.close(randomAccessFile);
      return null;
    }
    catch (Exception e) {
      FileUtils.close(randomAccessFile);
      return null;
    }
  }

  /**
   * Escreve uma mensagem de log de nvel grave.
   * 
   * @param text mensagem.
   */
  private void logSevere(String text) {
    CSDKLogger logger = CSDKLogger.getInstance();
    logger.logSevere(text);
  }

  /**
   * Libera o lock.
   * 
   * @param fileLock o lock.
   * @throws IOException em caso de erro ao liberar o lock;
   */
  private void _releaseLock(final FileLock fileLock) throws IOException {
    if (fileLock != null && fileLock.isValid()) {
      fileLock.release();
    }
  }

}
